import requests
import pandas as pd

class Farfetcher:
    """ API object to fetch listsings."""
    def __init__(self):
        self.base_url = r'https://www.farfetch.com/plpslice/listing-api/products-facets'
        self.headers = {
            'Origin': 'https://www.farfetch.com',
            'Referer': 'https://www.farfetch.com/shopping/m/items.aspx',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) '
                          'AppleWebKit/537.36 (KHTML, like Gecko) '
                          'Chrome/75.0.3770.100 Safari/537.36',
        }
        self.params = '?view={}&pagetype={}&pricetype={}&page={}&gender={}'

    def build_url(self, page=None, view=180, gender='Women', page_type='Shopping', price_type='FullPrice'):
        return self.base_url + self.params.format(view, page_type, price_type, page)

    def listings(self, page=1, gender='Women'):
        response = requests.get(self.build_url(page=page, gender=gender),
                                headers=self.headers)
        return response.json()

    def listings_to_products_dataframe(self, reponse_json):
        return pd.DataFrame.from_dict(response_json['products'])


"""
pagetype

Sets
Shopping
Stories

gender
Women
Men
Kids
"""




import sys
from datetime import date

import click
from tqdm import tqdm
import pandas as pd

from farfetcher import Farfetcher

CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])


@click.group(context_settings=CONTEXT_SETTINGS)
@click.version_option()
def cli():
    pass

@cli.command("crawl")
@click.option("--num-pages", "-n", default=180, help="No. of pages to crawl.")
@click.option("--gender", '-g', default="Women", help="Gender.")
@click.option("--depth", '-d', default=1, help="Depth of crawler.")
@click.option("--output-to", "-o", default="farfetch-{}-{}.tsv",
    help="Filename to save the dataframe."
)
def crawl_listings(num_pages, gender, depth, output_to):
    today = date.today()
    output_filename = output_to.format(gender, today.isoformat())
    ff = Farfetcher()
    all_response_jsons = []
    try:
        for i in tqdm(range(1,num_pages+1)):
            all_response_jsons.append(ff.listings(page=i, gender=gender))
    except KeyboardInterrupt:
        pass
    df = pd.concat([pd.DataFrame.from_dict(d['products']) for d in all_response_jsons])
    df.to_csv(output_filename, sep='\t', index=False)
    print(f"TSV file saved to {output_filename}",  file=sys.stderr)



from distutils.core import setup
import setuptools

console_scripts = """
[console_scripts]
farfetcher=farfetcher.cli:cli
"""

setup(
  name = 'farfetcher',
  packages = ['farfetcher'],
  version = '0.0.6',
  description = '',
  long_description = '',
  author = '',
  license = 'MIT',
  url = 'https://github.com/alvations/farfetcher',
  keywords = [],
  classifiers = [],
  install_requires = ['pandas', 'requests', 'click', 'tqdm'],
  entry_points=console_scripts,
)
